import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:multimedia_gallery/video/widget/video_overlay.dart';
import 'package:multimedia_gallery/video/widget/video_slider.dart';
import 'package:video_player/video_player.dart';

class VideoViewer extends StatefulWidget {
  const VideoViewer(
      {super.key,
      required this.uri,
      this.sliderStyle,
      this.timeStampStyle,
      this.onChangeEnd,
      this.onSliderChange,
      this.videoAspectRatio,
      this.onVideoScreenTap});

  final String uri;
  final SliderThemeData? sliderStyle;
  final TextStyle? timeStampStyle;
  final Function(double)? onSliderChange;
  final Function(double)? onChangeEnd;
  final double? videoAspectRatio;
  final void Function()? onVideoScreenTap;

  @override
  State<VideoViewer> createState() => _VideoViewerState();
}

class _VideoViewerState extends State<VideoViewer> {
  late VideoPlayerController controller;
  late Future<void> initializeVideoPlayerFuture;
  late Timer videoTimestamp;
  Timer disableShowIconTimer = Timer.periodic((const Duration(seconds: 5)), (timer) => {});
  bool isShowIcon = false;
  double sliderValue = 0;
  Duration currentPosition = Duration.zero;

  @override
  void initState() {
    super.initState();
    controller = VideoPlayerController.networkUrl(
      Uri.parse(widget.uri),
    );
    initializeVideoPlayerFuture = controller.initialize();
    playVideo();
    startTimer();
  }

  @override
  void dispose() {
    controller.dispose();
    disableShowIconTimer.cancel();
    videoTimestamp.cancel();
    super.dispose();
  }

  void startTimer() {
    videoTimestamp = Timer.periodic(const Duration(seconds: 1), (timer) {
      if (controller.value.position.inSeconds < controller.value.duration.inSeconds) {
        setState(() {
          sliderValue = controller.value.position.inSeconds.toDouble();
          currentPosition = controller.value.position;
        });
      } else {
        onVideoCompleted();
        currentPosition = controller.value.duration;
        timer.cancel();
      }
    });
  }

  void onVideoCompleted() {
    setState(() {
      sliderValue = controller.value.duration.inSeconds.toDouble();
    });
  }

  void _seekTo(double value) {
    setState(() {
      controller.seekTo(Duration(seconds: value.toInt()));
      sliderValue = value;
    });
  }

  void playVideo() {
    controller.play();
  }

  void pauseVideo() {
    controller.pause();
  }

  String formatDuration(Duration duration) {
    String twoDigits(int n) => n.toString().padLeft(2, '0');
    String minutes = duration.inMinutes.remainder(60).toString();
    String twoDigitSeconds = twoDigits(duration.inSeconds.remainder(60));
    return '$minutes:$twoDigitSeconds';
  }

  @override
  Widget build(BuildContext context) {
    SystemChrome.setPreferredOrientations([
      DeviceOrientation.portraitUp,
      DeviceOrientation.portraitDown,
      DeviceOrientation.landscapeLeft,
      DeviceOrientation.landscapeRight
    ]);
    return OrientationBuilder(builder: (context, orientation) {
      final isPortrait = orientation == Orientation.portrait;
      return Scaffold(
        backgroundColor: Colors.white,
        appBar: isPortrait
            ? AppBar(
                leading: IconButton(onPressed: () => Navigator.pop(context), icon: const Icon(Icons.arrow_back)),
                backgroundColor: Colors.white)
            : null,
        body: FutureBuilder(
            future: initializeVideoPlayerFuture,
            builder: (context, snapshot) {
              return snapshot.connectionState == ConnectionState.done
                  ? isPortrait
                      ? buildPortraitVideo(isPortrait)
                      : buildLandscapeVideo(isPortrait)
                  : Container(
                      color: Colors.white,
                      alignment: Alignment.topCenter,
                      child: const Center(child: CircularProgressIndicator()));
            }),
      );
    });
  }

  Widget buildPortraitVideo(bool isPortrait) {
    return Container(
      alignment: Alignment.center,
      height: MediaQuery.of(context).size.height - MediaQuery.of(context).padding.top - kToolbarHeight,
      // Fill the screen with white color
      child: AspectRatio(aspectRatio: controller.value.aspectRatio, child: buildVideo(isPortrait)),
    );
  }

  Widget buildLandscapeVideo(bool isPortrait) {
    SystemChrome.setEnabledSystemUIMode(SystemUiMode.leanBack);
    return Stack(
      children: [
        buildVideo(isPortrait),
      ],
    );
  }

  Widget buildVideo(bool isPortrait) {
    return GestureDetector(
      onTap: widget.onVideoScreenTap ?? () {
        setState(() {
          isShowIcon = !isShowIcon;
          disableShowIconTimer = Timer.periodic(
              (const Duration(seconds: 5)),
                  (timer) => setState(() {
                isShowIcon = false;
              }));
        });
      },
      child: Stack(fit: StackFit.loose, children: [
        VideoPlayer(controller),
        Visibility(
          visible: isShowIcon,
          child: Stack(
            children: [
              VideoOverlay(
                  isPortrait: isPortrait,
                  controller: controller,
                  onPlayPressed: () {
                    setState(() {
                      if (controller.value.isPlaying) {
                        controller.pause();
                      } else {
                        controller.play();
                      }
                    });
                  }),
              Positioned(
                bottom: 10,
                right: 10,
                left: 10,
                child: VideoSlider(
                    data: widget.sliderStyle,
                    currentPosition: formatDuration(currentPosition),
                    videoDuration: formatDuration(controller.value.duration),
                    child: Slider(
                        value: sliderValue,
                        min: 0,
                        max: controller.value.duration.inSeconds.toDouble(),
                        onChanged: widget.onSliderChange ??
                                (value) {
                              setState(() {
                                sliderValue = value;
                              });
                            },
                        onChangeEnd: widget.onChangeEnd ??
                                (value) {
                              _seekTo(value);
                            })),
              )
            ],
          ),
        ),

      ]),
    );
  }
}
